@==== Total number of repetitions == NCREP + NSKIP*NEREP ====@
new;
screen on;
output file=hbout1.txt reset;

@ Number of iterations to use prior to convergence @
NCREP = 10000;

@ Number of iterations to keep per respondent for averaging @
@ after convergence has been reached @
NEREP = 1000;

@ Number of iterations to do in between collecting estimates of @
@ the parameters @
NSKIP = 10;

@ How frequently to print info about the iteration process @
@ eg 10 == every 10 iterations print info @
INFOSKIP = 500;

@Want full or diagonal covariance matrix for random coefficients?@
@ Set FULLCV=1 for full cov matrix, FULLCV=0 for diagonal matrix.@
FULLCV=0;

@Do you want to save the means of A,D and B to a file? @
@If so, set KEEPMN=1. If no, set KEEPMN=0.@
KEEPMN=0;

@ If KEEPMN=1, specify ile to which to print the means.@
@ It will print the mean vector, then the variance-covariance matrix,@
@ and then the betas to this file.@
FOUTPUT = "hbbetas";

SEED1 = 4239;

NALT = 4;        @Number of alternatives@
NP = 361;        @Number of people@
NOBS = 4308;     @Number of choice situations (observations)@

NIV = 6;         @Number of explanatory variables to use in model.@
IDC = {1, 2, 3, 4, 6, 7};  @Location of expl vars in XMAT@

@================  DATA ================@
load XMAT[NOBS, 28] = xdata;    @Variables that can enter the model.@
                                @Each var is NALT cols wide.@
                                @User specifies in IDC above which to enter.@
@XMAT = XMAT[.,1:16] ~ XMAT[.,21:28]; @@Omit variable that is not used.@

load TIMES[NP,1] = times;       @Number of observations for each person.@

load YVEC[NOBS,1] = ydata;      @Chosen alternative for each observation.@
YPERM = zeros(NOBS,NALT);       @Make matrix of 0-1's to id the chosen alt.@
i = 1;
do while i<=NOBS;
  YPERM[i,YVEC[i]] = 1;
  i = i + 1;
endo;
clear YVEC;

@=========================================@

A = zeros(NIV,1);
B = zeros(NP,NIV);
D = NIV*eye(NIV);

@Starting values@
A={-.857, -0.183, 2.0977, 1.5247, -8.285, -8.53};
A=A .* 0.5;
B=1+B;
B=B .* A';

@================================================================@
SQNP = sqrt(NP);

@ This is a global variable to be used in proc nextBeta().         @
@ RHO is the proportionality fraction for the jumping distribution @
RHO = 0.1;

if FULLCV .eq 1;
   Print "This run specifies correlated betas (ie full cov matrix).";
elseif FULLCV .eq 0;
   Print "This run specifies uncorrelated betas (ie diagonal cov matrix).";
endif;

print "Number of explanatory variables is: " NIV;
print "The explanatory variables are: " IDC' " within XMAT." ;



format /rds 3,8;

{A, B, D} = hb(A, B, D);

print "A, meanc(B), stdc(B), sqrt(diag(D)";
print (A~meanc(B)~stdc(B)~sqrt(diag(D)));
print "D";
print D;
print "vcx(B)";
print vcx(B);

if KEEPMN .eq 1;
   screen off;
   output file=^FOUTPUT reset;
   A;
   D;
   B;
endif;


proc (3) = hb(a, b, d);
  local n, r, ma, mb, md, accept, p;

  rndseed SEED1;

  p = logit(b);

  @ Iterate until convergence @
  r = 1;
  do while r <= NCREP;
    {b, accept, p} = nextBeta(a, b, d, p);
    a = nextAlpha(b, d);
    if FULLCV;
           d = nextVar(a, b);
    else;
           d = nextVarConstrain(a, b);
     endif;

    if((r % INFOSKIP) == 0);
      print "Iteration:     RHO      Accept Rate";
      print (r~RHO~accept);
      print "Means of A and B:";;
      print (a' | meanc(b)');
      print;
    endif;

    r = r + 1;
  endo;

  print "Obtained Converged Parameters.  Starting Averaging Iterations.";
  print;

  ma = zeros(NIV, 1);
  mb = zeros(NP, NIV);
  md = zeros(NIV, NIV);

  @ Iterate after convergence collecting averages @
  n = NEREP*NSKIP;
  r = 1;
  do while r <= n;
    {b, accept, p} = nextBeta(a, b, d, p);
    a = nextAlpha(b, d);
    if FULLCV; 
            d = nextVar(a, b);
       else; 
            d = nextVarConstrain(a, b);
   endif;

    if((r % NSKIP) == 0);
      ma = ma + a;
      mb = mb + b;
      md = md + d;
    endif;

    if((r % INFOSKIP) == 0);
      print "Iteration:     RHO      Accept Rate";
      print (r~RHO~accept);
      print "Means of A and B:";;
      print (a' | meanc(b)');
      print;
    endif;

    r = r + 1;
  endo;

  retp(ma/NEREP, mb/NEREP, md/NEREP);
endp;

proc nextAlpha(b, d);
  retp(meanc(b) + (chol(d)')*rndn(NIV,1)/SQNP);
endp; 

proc nextVar(a, b);
  local d, k, s;

  b = a' - b;
  d = zeros(NIV, NIV);
  k = 1;
  do while k <= NIV;
    d[.,k] = sumc(b .* b[.,k]);
    d[k,k] = d[k,k] + NIV;
    k = k + 1;
  endo;

  s = ((chol(inv(d))') * rndn(NIV, NP+NIV))';
  k = 1;
  do while k <= NIV;
    d[.,k] = sumc(s .* s[.,k]);
    k = k + 1;
  endo;

  retp(inv(d));
endp;

@ Only iterates over the diagonal elements @
proc nextVarConstrain(a, b);
  local d, k, s, t;

  d = zeros(NIV, NIV);
  b = a' - b;

  k = 1;
  do while k <= NIV;

    t = 1 + b[.,k]'b[.,k];
    s = sqrt(1/t) * rndn(NP+1, 1);
    d[k,k] = inv(s's);

    k = k + 1;
  endo;

  retp(d);
endp;

proc (3) = nextBeta(a, bold, d, pold);
  local pnew, bnew, bn, bo, r, nind, ind, i;

  bnew = bold + rndn(NP,NIV) * chol(d) * sqrt(RHO);

  bn = bnew - a';
  bo = bold - a';
  pnew = logit(bnew);
  r = pnew./pold .* exp(-0.5 * (sumc(bn'.*(inv(d)*bn')) -
                                sumc(bo'.*(inv(d)*bo')) ));
  ind = (r .GE 1) + (r .LT 1).*(rndu(NP,1) .LE r);

  i = sumc(ind)/NP;
  if(i < 0.3);
    RHO = RHO - 0.001;
  elseif(i > 0.3);
    RHO = RHO + 0.001;
  endif;

  nind = ind .EQ 0;
  retp((ind.*bnew + nind.*bold), i, (ind.*pnew + nind.*pold));
endp;

proc logit(b);
  local p0, bg, en, n, v, k, km;

  p0 = zeros(NP,1);

  en = 0;
  n = 1;
  do while n <= NP;
    bg = en + 1;
    en = en + TIMES[n];

    v = zeros(TIMES[n], NALT);
    k = 1;
    do while k <= NIV;
      km = (IDC[k,1]-1)*NALT;
      v = v + XMAT[bg:en, (km+1):(km+NALT)] * b[n,k];
      k = k + 1;
    endo;
    v = exp(v);

    p0[n] = prodc( sumc((v.*YPERM[bg:en,.])') ./ sumc(v') );

    n = n + 1;
  endo;

  retp(p0);
endp;


